//欧几里得算法 求两个数a、b的最大公约数
function gcd(a,b){
    return b===0?a:gcd(b,a%b)
}
//分数类 分子，分母
class Fraction{
    static create(num,den=1) {
        if(num instanceof Fraction){
            return num;
        }else if(/(-?\d+)\/(\d+)/.test(num)){
            return new Fraction(parseInt(RegExp.$1),parseInt(RegExp.$2))
        }else{
            if(/\.(\d+)/.test(num)){
                num=num*Math.pow(10,RegExp.$1.length);
                den=den*Math.pow(10,RegExp.$1.length);
            }
            if(/\.(\d+)/.test(den)){
                num=num*Math.pow(10,RegExp.$1.length);
                den=den*Math.pow(10,RegExp.$1.length);
            }
            return new Fraction(num,den)
        }
    }
    constructor(num=0,den=1){
        if(den<0){
            num=-num;
            den=-den;
        }
        if(den===0){
            throw '分母不能为0'
        }
        let g=gcd(Math.abs(num),den)
        this.num=num/g;
        this.den=den/g;
    }
    //加
    add(o){
        return new Fraction(this.num*o.den+this.den*o.num,this.den*o.den)
    }
    //减
    sub(o){
        return new Fraction(this.num*o.den-this.den*o.num,this.den*o.den)
    }
    //乘
    multiply(o){
        return new Fraction(this.num*o.num,this.den*o.den);
    }
    //除
    divide(o){
        return new Fraction(this.num*o.den,this.den*o.num);
    }
    //小于
    lessThan(o){
        return this.num*o.den<this.den*o.num;
    }
    //等于
    equal(o){
        return this.num*o.den===this.den*o.num;
    }
    toString() {
        return this.num+'/'+this.den;
    }
}

//解析数学表达式
const execMathExpress = function(formula,obj){
    //局部变量
    const tempObj=Object.assign({
        _0:0
    },obj);
    //计算缓存
    const keyCache={};
    let index=1;

    formula=formula.replace(/ /g,'');//清理空格
    //解析数字
    formula=formula.replace(/(^|[(*+/-])(\d+\.\d+|\d+)/g,function (m,p1,p2) {
        if(keyCache[p2]){
            return p1+keyCache[p2];
        }
        const key=keyCache[p2]='_'+index++;
        tempObj[key]=Fraction.create(p2);
        return p1+key;
    })

    function getKey(p1,p2,p3) {
        const keyC=p1+p2+p3;
        if(keyCache[keyC]){
            return keyCache[keyC];
        }
        const key=keyCache[keyC]='_'+index++;
        const fA=Fraction.create(tempObj[p1])
        const fB=Fraction.create(tempObj[p3])
        if(p2==='*'){
            tempObj[key]=fA.multiply(fB);
        }else if(p2==='/'){
            tempObj[key]=fA.divide(fB);
        }else if(p2==='+'){
            tempObj[key]=fA.add(fB);
        }else if(p2==='-'){
            tempObj[key]=fA.sub(fB);
        }
        return key;
    }
    function run(s) {

        //子表达式
        if(/\(([^\(]+?)\)/.test(s)){
            s=s.replace(/\(([^\(]+?)\)/g,function (m,p1,p2) {
                return run(p1);
            })
        }
        //负号
        s=s.replace(/([*/+]|^)-(\w+)/g,function (m,p1,p2) {
            return getKey('_0','-',p2);
        })
        //返回
        if(/(^\w+$)/.test(s)){
            return RegExp.$1;
        }
        //乘法、除法、加法、减法
        const expArr=['*','/','+','-'];
        for(let i=0;i<expArr.length;i++){
            const p=expArr[i];
            const reg=new RegExp('(\\w+)['+p+'](\\w+)');
            while (reg.test(s)){
                s=s.replace(reg,function (m,p1,p2) {
                    return getKey(p1,p,p2);
                })
            }
        }
        //返回
        if(/(^\w+$)/.test(s)){
            return RegExp.$1;
        }
        return run(s);
    }
    return tempObj[run(formula)]
}

const calcSumPrice = function(category){
	/**
	 * 帘布小计 = 帘布每米价格×窗宽×褶比
	 * 纱布小计 = 纱布每米价格×窗宽×褶比
	 * 辅料小计 = 辅料每米价格×窗宽×褶比
	 * 轨道小计 = 轨道每米价格×窗宽
	 * 花边小计 = 花边每米价格×(窗宽×褶比×2+窗高×4)
	 * 垂线小计 = 垂线每米价格×(窗宽×褶比)
	 * 安装小计 = 安装价格
	 */
	let X = 0.00;//单价
	let Y = 1.00;//占比
	let sumName = '';
	let unitName = '';
	let sumCount = 0;
	let sumPrice = 0.00;
	
	let purchase = category.purchases[0];
	
	let N1=0,P1=0,P2=0,P3=0,P4=0,P5=0,P6=0,P7=0,P8=0,P9=0;
	for (let j=0;j<purchase.length;j++) {
		let it = purchase[j];
		if (it.name=='N1') {
			N1 = it.value&&it.value>=1?it.value:1;
			sumCount = N1;
			unitName = it.unitName;
		}
		else if (it.name=='P1')
			P1 = it.value;
		else if (it.name=='P2')
			P2 = it.value;
		else if (it.name=='P3')
			P3 = it.value;
		else if (it.name=='P4')
			P4 = it.value;
		else if (it.name=='P5')
			P5 = it.value;
		else if (it.name=='P6')
			P6 = it.value;
		else if (it.name=='P7')
			P7 = it.value;
		else if (it.name=='P8')
			P8 = it.value;
		else if (it.name=='P9')
			P9 = it.value;
	}
	
	if (category.subunits)
	for (let j=0;j<category.subunits.length;j++) {
		let subunit = category.subunits[j];
		Y = subunit.rate;
		let exp = subunit.express;
		
		if (subunit.tinykinds)
		for (let k=0;k<subunit.tinykinds.length;k++) {
			
			let tinykind = subunit.tinykinds[k];
			
			X = tinykind.salesPrice;
			let res = this.execMathExpress(exp,{
				X: X,
				N1: N1,
				Y: Y,
				P1: P1,
				P2: P2,
				P3: P3,
				P4: P4,
				P5: P5,
				P6: P6,
				P7: P7,
				P8: P8,
				P9: P9
			});
			//console.log('res'+JSON.stringify(res))
			
			let subtotal = res.num / res.den;
			tinykind.subtotal = subtotal;
			
			if (subunit.checked && tinykind.checked) {
				let nnn = tinykind.name;
				//if (nnn.indexOf(subunit.name) < 0)
				//	nnn += subunit.name;
				sumName += nnn+',';
				sumPrice += tinykind.subtotal;
			}
			
		}
		
	}
	
	category.sumName = sumName;
	category.unitName = unitName;
	category.sumCount = sumCount;
	
	if (sumPrice > 0) {
		category.sumPrice = sumPrice;
		category.sumMoney = sumPrice * category.discount;
		category.unitPrice = category.sumPrice / category.sumCount;
		category.unitMoney = category.sumMoney / category.sumCount;
	} else {
		category.discount = 1;
		category.sumPrice = category.sumMoney;
		category.unitPrice = category.sumPrice / category.sumCount;
		category.unitMoney = category.sumMoney / category.sumCount;
	}
	category.sumPrice = parseFloat(parseFloat(category.sumPrice).toFixed(2));
	category.sumMoney = parseFloat(parseFloat(category.sumMoney).toFixed(2));
	category.unitPrice = parseFloat(parseFloat(category.unitPrice).toFixed(2));
	category.unitMoney = parseFloat(parseFloat(category.unitMoney).toFixed(2));
}

export default {
	execMathExpress,
	calcSumPrice,
}
